package com.ruoyi.ss.bonus.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.enums.ServiceType;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.bonus.service.BonusConverter;
import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.device.domain.enums.DeviceServiceMode;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.bo.RechargeBO;
import com.ruoyi.ss.transactionBill.domain.dto.RechargePayBO;
import com.ruoyi.ss.transactionBill.domain.vo.UserRechargeServiceVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author wjh
 * 2024/8/20
 */
@Service
public class BonusConverterImpl implements BonusConverter {

    @Autowired
    private ISmUserService userService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private ISysConfigService sysConfigService;

    /**
     * 订单转为分成明细
     *
     * @param bo
     */
    @Override
    public List<Bonus> toPo(RechargePayBO bo) {
        if (bo == null) {
            return Collections.emptyList();
        }

        TransactionBill bill = bo.getOrder();
        if (bill == null) {
            return Collections.emptyList();
        }

        List<Bonus> result = genBonusList(bo.getMch(), null, bo.getPlatform(), bo.getDevice(), bo.getChannel());

        for (Bonus bonus : result) {
            bonus.setBillId(bill.getBillId());
            bonus.setBillNo(bill.getBillNo());
        }

        return result;
    }

    @Override
    public List<Bonus> toPo(RechargeBO bo) {
        if (bo == null) {
            return Collections.emptyList();
        }

        TransactionBill bill = bo.getOrder();
        if (bill == null) {
            return Collections.emptyList();
        }

        List<Bonus> result = genBonusList(bo.getMch(), bo.getAgent(), bo.getPlatform(), bo.getDevice(), null);

        for (Bonus bonus : result) {
            bonus.setBillId(bill.getBillId());
            bonus.setBillNo(bill.getBillNo());
        }

        return result;
    }

    @Override
    public List<Bonus> genBonusList(SmUserVo mch, SmUserVo agent, SysDept platform, DeviceVO device, ChannelVO channel) {
        List<Bonus> result = new ArrayList<>();
        if (mch == null || platform == null || device == null) {
            return result;
        }
        // 剩余可分配的分成比例
        BigDecimal point = BigDecimal.valueOf(100);

        // 直营模式
        if (DeviceServiceMode.DIRECT.getMode().equals(device.getServiceMode())) {
            // 平台收取服务费
            UserRechargeServiceVO userRechargeService = transactionBillService.getMchRechargeService(channel, mch, device);
            String serviceType = userRechargeService.getServiceType();  // 服务费类型
            BigDecimal serviceRate = userRechargeService.getServiceRate();   // 服务费
            // 如果按照百分比收取服务费
            if(ServiceType.PERCENT.getType().equals(serviceType)) {
                result.add(this.toPo(platform, serviceRate));
                point = point.subtract(serviceRate);
            }
        }
        // 代理模式
        else if (DeviceServiceMode.AGENT.getMode().equals(device.getServiceMode())) {
            // 平台收取服务费
            ServiceUtil.assertion(agent.getAgentServiceRate() == null, "代理商服务费未配置");
            ServiceUtil.assertion(device.getAgentServiceRate() == null, "代理商设备服务费未配置");

            result.add(this.toPo(platform, agent.getAgentServiceRate()));
            point = point.subtract(agent.getAgentServiceRate());

            // 代理商收取服务费
            result.add(this.toPo(agent, device.getAgentServiceRate(), agent.getUserId().toString()));
            point = point.subtract(device.getAgentServiceRate());
        }

        ServiceUtil.assertion(point.compareTo(BigDecimal.ZERO) < 0, "商户剩余分成不允许小于0");
        ServiceUtil.assertion(point.compareTo(BigDecimal.valueOf(100)) > 0, "商户剩余分成不允许大于100");

        // 剩余的给商户
        String ancestors = Stream.of(device.getAgentId(), device.getUserId()).map(Object::toString).collect(Collectors.joining(","));
        result.add(this.toPo(mch, point, ancestors));

        return result;
    }

    private Bonus toPo(SysDept dept, BigDecimal point) {
        if (dept == null) {
            return null;
        }

        Bonus po = new Bonus();
        po.setStatus(BonusStatus.UN_DIVIDEND.getStatus());
        po.setArrivalId(dept.getDeptId());
        po.setArrivalName(dept.getDeptName());
        BonusArrivalType arrivalType = BonusArrivalType.parseByDept(dept);
        if (arrivalType != null) {
            po.setArrivalType(arrivalType.getType());
        }
        po.setPoint(point);
        po.setAncestors(dept.getAncestors());
        po.setDeptId(dept.getDeptId());

        return po;
    }

    private Bonus toPo(SmUserVo user, BigDecimal point, String ancestors) {
        if (user == null || point == null) {
            return null;
        }

        Bonus po = new Bonus();
        po.setStatus(BonusStatus.UN_DIVIDEND.getStatus());
        po.setArrivalId(user.getUserId());
        po.setArrivalName(user.getRealOrUserName());
        BonusArrivalType arrivalType = BonusArrivalType.parseByUserType(user.getType());
        if (arrivalType != null) {
            po.setArrivalType(arrivalType.getType());
        }
        po.setPoint(point);
        po.setAncestors(ancestors);
        po.setDeptId(Constants.ROOT_DEPT);

        return po;
    }
}
